// AvlTree class
// extends binary search tree to be an avl tree.
// written by Ed Hong, 11/09/04, partly based on code
//   initially written by Mark Allen Weiss and Donald Chinn.
public class AvlTree extends BinarySearchTree {

   //================================================================
   //  Creates an empty AVL tree.
   //================================================================
   public AvlTree() 
   {
      super();
   }  // constructor AVLTree

   //================================================================
   //  Creates an AVL tree with the specified element as its
   //  root.
   //================================================================
   public AvlTree (Comparable element) 
   { // always use AvlNodes instead of BinaryTreeNodes
      count = 1;
	  root = (BinaryTreeNode) new AvlNode(element);
   }  // constructor AVLTree

   //================================================================
   //  Constructs an AVL tree from the two specified AVL trees.
   //================================================================
   public AvlTree (Object element, AvlTree leftSubtree,
                                      AvlTree rightSubtree) 
   { 
     System.out.println("Error, this constructor is not implemented");
   }

   //================================================================
   //  Removes the left subtree of this Avl tree.
   //================================================================
   public void removeLeftSubtree() 
   {
      super.removeLeftSubtree();
	  // update the height. 
	  ((AvlNode) root).height = 1 + height((AvlNode) root.right);
	  
   }  // method removeLeftSubtree

   //================================================================
   //  Removes the right subtree of this avl tree.
   //================================================================
   public void removeRightSubtree() 
   {
	  super.removeRightSubtree();
	  // update the height. 
	  ((AvlNode) root).height = 1 + height((AvlNode) root.left);
	  
   }  // method removeRightSubtree
   

   //================================================================
   //  Adds the specified object to the AVL tree in the
   //  appropriate position according to its key value.  Note that
   //  equal elements are not added but are ignored.
   //================================================================
    public void addElement( Comparable x ) {
	    System.out.println("In AVLtree addElement(" + x + ")");
        root = insert( x, (AvlNode) root );
    }

    //================================================================
   //  Removes the first element that matches the specified target
   //  element from the AVL tree and returns a reference to
   //  it.  Throws a ElementNotFoundException if the specified target
   //  element is not found in the AVL tree.
   //================================================================
   public Comparable removeElement (Comparable targetElement) throws
     ElementNotFoundException {
      System.out.println(
	    "AVL remove not implemented; using BST remove (may unbalance the tree).");
	  return super.removeElement(targetElement);
   }
 
   //================================================================
   //  Removes the node with the least value from the AVL
   //  tree and returns a reference to its element.  Throws an
   //  EmptyBinarySearchTreeException if the AVL tree is
   //  empty. 
   //================================================================
   public Comparable removeMin() throws EmptyCollectionException 
   {
      System.out.println(
	    "AVL remove not implemented; using BST remove (may unbalance the tree).");
	  return super.removeMin();
   }

  //================================================================
   //  Removes the node with the highest value from the AVL
   //  tree and returns a reference to its element.  Throws an
   //  EmptyBinarySearchTreeException if the AVL tree is
   //  empty. 
   //================================================================
   public Comparable removeMax() throws EmptyCollectionException 
   {
      System.out.println(
	    "AVL remove not implemented; using BST remove (may unbalance the tree).");
	  return super.removeMax();
   }

    /**
     * Internal method to get element field.
     * @param t the node.
     * @return the element field or null if t is null.
     */
    private Comparable elementAt( AvlNode t ) {
        return t == null ? null : (Comparable) t.element;
    }

    /**
     * Internal method to insert into a subtree.
     * @param x the item to insert.
     * @param t the node that roots the tree.
     * @return the new root.
     */
    private AvlNode insert( Comparable x, AvlNode t ) {
		return null;
    }
    
    /**
     * Internal method to print a subtree rooted at t,
     * indicating depth.
     * @param t the node that roots the (sub)tree
     * @param depth the current depth to print the subtree
     */
    private void printTreeDepth( AvlNode t, int depth ) {
        if( t != null ) {
            printTreeDepth( (AvlNode) t.right, depth + 1 );
            for (int i = 0; i < depth; i++) {
                System.out.print("  ");
            }
            System.out.println(t.element);
            printTreeDepth( (AvlNode) t.left, depth + 1 );
        }
    }


    /**
     * Return the height of node t, or -1, if null.
	 * 
     */
    private static int height( AvlNode t ) {
        return t == null ? -1 : t.height;
    }

    /**
     * Return maximum of lhs and rhs.
     */
    private static int max( int lhs, int rhs ) {
        return lhs > rhs ? lhs : rhs;
    }

    /**
     * Rotate binary tree node with left child.
     * For AVL trees, this is a single rotation for case 1.
     * Update heights, then return new root.
     */
    private static AvlNode rotateWithLeftChild( AvlNode k2 ) {
		return null;
    }

    /**
     * Rotate binary tree node with right child.
     * For AVL trees, this is a single rotation for case 4.
     * Update heights, then return new root.
     */
    private static AvlNode rotateWithRightChild( AvlNode k1 ) {
		return null;
    }

    /**
     * Double rotate binary tree node: first left child
     * with its right child; then node k3 with new left child.
     * For AVL trees, this is a double rotation for case 2.
     * Update heights, then return new root.
     */
    private static AvlNode doubleWithLeftChild( AvlNode k3 ) {
		return null;
    }
    
    /**
     * Double rotate binary tree node: first right child
     * with its left child; then node k1 with new right child.
     * For AVL trees, this is a double rotation for case 3.
     * Update heights, then return new root.
     */
    private static AvlNode doubleWithRightChild( AvlNode k1 ) {
		return null;
    }
	
    
    // a test program
    public static void test () {
        AvlTree avltree = new AvlTree();
        
        avltree.addElement(new String("Happy"));
        avltree.addElement(new String("weird"));
        avltree.addElement(new String("sad"));
        avltree.addElement(new String("Acropolis"));
        avltree.addElement(new String("ZZZZZZYY"));
        avltree.printTreeDepth((AvlNode) avltree.root, 0);
    }
    
    // Test program
    public static void main( String [ ] args ) {
	    test();
        AvlTree t = new AvlTree( );
        final int NUMS = 4000;
        final int GAP  =   37;
        System.out.println( "Checking... (no more output means success)" );

        for( int i = GAP; i != 0; i = ( i + GAP ) % NUMS )
            t.addElement( new Integer( i ) );
        if( NUMS < 40 )
            System.out.println(t);
        if( ((Integer)(t.findMin( ))).intValue( ) != 1 ||
            ((Integer)(t.findMax( ))).intValue( ) != NUMS - 1 )
            System.out.println( "FindMin or FindMax error!" );

        for( int i = 1; i < NUMS; i++ )
             if( ((Integer)(t.find( new Integer( i ) ))).intValue( ) != i )
                 System.out.println( "Find error1!" );
    }
    

}
