//*******************************************************************
//
// BinarySearchTree.java			Authors: Lewis/Chase
//
// Modified by edhong; now ignores duplicate elements.
// Implements the BinarySearchTreeADT interface with links
//*******************************************************************

// package jss2; modified by edhong

// import jss2.exceptions.*;
// import jss2.*;

public class BinarySearchTree extends BinaryTree implements BinarySearchTreeADT {

   //================================================================
   //  Creates an empty binary search tree.
   //================================================================
   public BinarySearchTree()
   {
      super();
   }  // constructor BinarySearchTree

   //================================================================
   //  Creates a binary search with the specified element as its
   //  root.
   //================================================================
   public BinarySearchTree (Comparable element)
   {
      super (element);
   }  // constructor BinarySearchTree

   //================================================================
   //  Adds the specified object to the binary search tree in the
   //  appropriate position according to its key value.  Note that
   //  equal elements are not added but are ignored.
   //================================================================
   public void addElement (Comparable element)
   {

      BinaryTreeNode temp = new BinaryTreeNode (element);

      if (isEmpty())
         root = temp;
      else
      {
         BinaryTreeNode current = root;
         boolean added = false;

         while (!added)
         {
            if (element.compareTo(current.element) < 0)

               if (current.left == null)
               {
                  current.left = temp;
                  added = true;
               }
               else
                  current = current.left;
            else if (element.compareTo(current.element) > 0)

               if (current.right == null)
               {
                  current.right = temp;
                  added = true;
               }
               else
                  current = current.right;
			else
			   return;
         }//while
      }//else

      count++;

   }  // method addElement

   //================================================================
   //  Removes the first element that matches the specified target
   //  element from the binary search tree and returns a reference to
   //  it.  Throws a ElementNotFoundException if the specified target
   //  element is not found in the binary search tree.
   //================================================================
   public Comparable removeElement (Comparable targetElement) throws
   ElementNotFoundException
   {
      Comparable result = null;

      if (!isEmpty())

         if (targetElement.equals(root.element))
         {  // remove root node
            result = (Comparable) root.element;
            root = replacement (root);
            count--;
         } //if
        else
        { // not at root, traverse down the tree one step
            BinaryTreeNode current; // this points to the current node
			BinaryTreeNode parent = root; // points to parent of current
            boolean found = false;

            if (targetElement.compareTo(root.element) < 0)
               current = root.left;
            else
               current = root.right;
            // traverse down the tree until we find the target,
			// or until target is known not to be in the list
            while (current != null && !found)
            {
               if (targetElement.equals(current.element))
               {// found the item; remove it.
                  found = true;
                  count--;
                  result = (Comparable) current.element;

                  if (current == parent.left)
                     parent.left = replacement (current);
                  else
                     parent.right = replacement (current);
               } //if
              else
              { // continue traversal down tree
                  parent = current;

                  if (targetElement.compareTo(current.element) < 0)
                     current = current.left;
                  else
                     current = current.right;
               } //else
            } //while
            if (!found)
               throw new ElementNotFoundException("binary tree");
         } //else

      return result;

   }  // method removeElement

   //================================================================
   //  Removes elements that match the specified target
   //  element from the binary search tree
   //  Throws a ElementNotFoundException if the sepcified target
   //  element is not found in the binary search tree.
   //================================================================
   public void removeAllOccurrences (Comparable targetElement) throws
   ElementNotFoundException
   {
      removeElement(targetElement);

      try
	 {
	   while (contains(targetElement))
          removeElement(targetElement);
	 }
	 catch (Exception ElementNotFoundException)
	 {
	 }

   }  // method removeAllOccurrences

   //================================================================
   //  Removes the node with the least value from the binary search
   //  tree and returns a reference to its element.  Throws an
   //  EmptyBinarySearchTreeException if the binary search tree is
   //  empty.
   //================================================================
   public Comparable removeMin() throws EmptyCollectionException
   {
      Comparable result = null;

      if (isEmpty())
           throw new EmptyCollectionException ("binary tree");
      else
      {
         if (root.left == null)
         {
            result = (Comparable) root.element;
            root = root.right;
         } //if
         else
         {
            BinaryTreeNode parent = root;
            BinaryTreeNode current = root.left;
            while (current.left != null)
            {
               parent = current;
               current = current.left;
            } //while
            result = (Comparable) current.element;
            parent.left = current.right;
         } //else

         count--;
      } //else

      return result;

   }  // method removeMin

   //================================================================
   //  Removes the node with the highest value from the binary
   //  search tree and returns a reference to its element.  Throws an
   //  EmptyBinarySearchTreeException if the binary search tree is
   //  empty.
   //================================================================
   public Comparable removeMax() throws EmptyCollectionException
   {
      Comparable result = null;

      if (isEmpty())
           throw new EmptyCollectionException ("binary tree");
      else
      {
         if (root.right == null)
         {
            result = (Comparable) root.element;
            root = root.left;
         } //if
         else
         {
              BinaryTreeNode parent = root;
              BinaryTreeNode current = root.right;

              while (current.right != null)
              {
                 parent = current;
                 current = current.right;
              } //while

              result = (Comparable) current.element;
              parent.right = current.left;
           } //else

         count--;
      } //else

      return result;

   }  // method removeMax

   //================================================================
   //  Returns the element with the least value in the binary search
   //  tree.  It does not remove the node from the binary search
   //  tree.  Throws an EmptyBinarySearchTreeException if the binary
   //  search tree is empty.
   //================================================================
   public Comparable findMin() throws EmptyCollectionException
   {
      Comparable result = null;

      if (isEmpty())
           throw new EmptyCollectionException ("binary tree");
      else
      {
         BinaryTreeNode current = root;

         while (current.left != null)
            current = current.left;

         result = (Comparable) current.element;
      } //else

      return result;

   }  // method findMin

   //================================================================
   //  Returns the element with the highest value in the binary
   //  search tree.  It does not remove the node from the binary
   //  search tree.  Throws an EmptyBinarySearchTreeException if the
   //  binary search tree is empty.
   //================================================================
   public Comparable findMax() throws EmptyCollectionException
   {
      Comparable result = null;

      if (isEmpty())
           throw new EmptyCollectionException ("binary tree");
      else
      {
         BinaryTreeNode current = root;

         while (current.right != null)
            current = current.right;

        result = (Comparable) current.element;
      } //else

      return result;

   }  // method findMax

   //================================================================
   //  Returns a reference to the specified target element if it is
   //  found in the binary tree.  Throws a NoSuchElementException if
   //  the specified target element is not found in the binary tree.
   //================================================================
   public Object find (Comparable targetElement) throws ElementNotFoundException
   {
	if (isEmpty())
        throw new ElementNotFoundException ("binarytree");

	 BinaryTreeNode current = root;
	 BinaryTreeNode temp = current;

      if (!(current.element.equals(targetElement)) && (current.left !=null)&&(((Comparable)current.element).compareTo(targetElement) > 0))
		current = findagain( targetElement, current.left);

      else if (!(current.element.equals(targetElement)) && (current.right != null))
		current = findagain( targetElement, current.right);

      if (!(current.element.equals(targetElement)))
         throw new ElementNotFoundException ("binarytree");

      return current.element;

   }  // method find

   //================================================================
   //  Returns a reference to the specified target element if it is
   //  found in the binary tree.
   //================================================================
   private BinaryTreeNode findagain (Comparable targetElement, BinaryTreeNode next)
   {
	 BinaryTreeNode current = next;
      if (!(next.element.equals(targetElement)) && (next.left !=null) &&(((Comparable)next.element).compareTo(targetElement) > 0))
 		next = findagain( targetElement, next.left);
	 else if (!(next.element.equals(targetElement)) && (next.right != null))
 		next = findagain( targetElement, next.right);

	 return next;

   }  // method findagain


   //================================================================
   //  Returns a reference to a node that will replace the one
   //  specified for removal.  In the case where the removed
   //  node has two children, the inorder successor is used
   //  as its replacement.
   //================================================================
   protected BinaryTreeNode replacement (BinaryTreeNode node)
   {

      BinaryTreeNode result = null;

      if ((node.left == null)&&(node.right==null))
            result = null;
      else if ((node.left != null)&&(node.right==null))
            result = node.left;
      else if ((node.left == null)&&(node.right != null))
            result = node.right;
      else
      {
            BinaryTreeNode current = node.right;
            BinaryTreeNode parent = node;

            while (current.left != null)
            {
               parent = current;
               current = current.left;
            }//while

            parent.left = current.right;
            result = current;
      }//else

      return result;

   }  // method replacement

}  // class BinarySearchTree

