// Written by Ed Hong, 11/09/04.
// Uses a BinarySearchTree to implement the SimpleMap interface.
// SimpleMap is an extremely simplified version of
//  Java's Map interface.
// This class will map keys to values by storing
//  <key,value> pairs inside a binary search tree; the pairs
//  are sorted by key.

import java.util.*;

public class BinarySearchTreeMap implements SimpleMap {
    BinarySearchTree tree; // tree will store (key, value) pairs.

    // Constructor; simply initializes the binary search tree.
    public BinarySearchTreeMap () {
	    tree = new BinarySearchTree();
	}

	// New java object for storing mappings.
	protected class KeyValuePair implements Comparable {
	    Comparable key; // the key
		Object value; // the value

	    // constructor for new object storing (k,v)
		public KeyValuePair(Comparable k, Object v) {
		  key = k;
		  value = v;
		}

		// updates value instance variable to input parameter
		public void replaceValue(Object v) {
		   value = v;
		}

		// accessor for value
		public Object getValue() {
		   return value;
		}

		// accessor for key.
		public Object getKey() {
		   return key;
		}

		// by default, natural comparison ordering is by key.
		public int compareTo(Object o) {
		   KeyValuePair x = (KeyValuePair) o;
		   // null key values precede non-null keys.
		   if (key == null)  {
		       if (x.getKey() == null)
			      return 0;
			   else // this.key is null, o.key is not
			      return -1;
		   }
		   else if (x.getKey() == null) //o.key is null, this.key is not
		       return 1;
		   else // default comparison of two non-null keys.
		       return this.key.compareTo(x.getKey());
		}

		// equals between two KeyValuePairs
		// is true whenever the two KeyValuePairs have equal keys.
		public boolean equals(Object o) {
		   KeyValuePair x = (KeyValuePair) o;
		   if (this.key == null) {
			   if (x.getKey() == null)
			   // null keys are equal to null keys.
			        return true;
               // null keys are not equal to non-null keys
			   else return false;
		   }
		   else if (x.getKey() == null)
		       // here this.key is nonnull, but o.key is null
			   return false;
		   // default equals comparison of two non-null keys.
		   return this.key.equals(x.getKey());
		}

		// Returns a string representation of this KeyValuePair
		// looks like <key,value>
		public String toString() {
		   String keyString, valueString;
		   if (key==null) keyString = "null";
		   else keyString = key.toString();
		   if (value == null) valueString = "null";
		   else valueString = value.toString();
		   return "<" + keyString+ "," + valueString + ">";
		}
	}


/*	Returns the value to which this map maps the specified key.
    Returns null if the map contains no mapping for this key.
	In other words, this method finds the KeyValuePair stored
	in tree whose key field .equals the specified key and returns
    the value of the key.
	You may assume key != null.
  Parameters:
      key - key whose associated value is to be returned.
  Returns:
      the value to which this map maps the specified key, or
	   null if the map contains no mapping for this key.*/
	// You should implement this with an appropriate call to a method of the
	// binarysearchtree. followed by an appropriate return.
	public Object get(Comparable key) {
	   System.out.println("Error, not implemented");
	   return null;
	}

/*  Associates the specified value with the specified key in this map.
    If the map previously contained a mapping for this key,
	the old value is replaced by the specified value.
	You may assume key != null and value != null.
  Parameters:
    key - key with which the specified value is to be associated.
    value - value to be associated with the specified key.
*/
	// You should implement this with an appropriate call to a method of the
	// binarysearchtree, followed by an appropriate value updating call.
	public void put(Comparable key, Object value) {
	   System.out.println("Error, not implemented");
	}


	// Prints out the first 10 KeyValuePairs in binary search tree in
	// sorted order. If there are fewer than 10 items, prints
	// all of them out and stops.
	public void printFirst10() {
	   System.out.println("Error, not implemented");
	}

	// Prints out the most frequently occuring 10 KeyValuePairs
	// in binary tree in sorted order.
	// If there are fewer than 10 items, prints
	// all of them out and stops.
	// You should write your code so that it is very easy to
	// change the number of words printed out.
	public void print10MostFrequent() {
       System.out.println("Error, not implemented");
	}
}
