/**
	A Quadratic Probing Hash Table
	  implementation of the SimpleMap interface. 
		    
    Created by Ed Hong, based on initial code by Mark Allen Weiss.
    November 20, 2004

*/

//
//
// ******************PUBLIC OPERATIONS*********************
// void put(k, v)       --> Insert (k,v) into table; hashCode based on k,
// Object get( k )      --> Return element v associated with key k. 
// void makeEmpty( )      --> Remove all items
//
// (optional operation)
// void remove( x )       --> Remove x

/**
 * Probing table implementation of hash tables.
 * Note that all "matching" is based on the equals method.
 */
public class QuadraticProbingHashTableMap implements SimpleMap {
    /**
     * Construct the hash table.
     */
    public QuadraticProbingHashTableMap( ) {
        this( DEFAULT_TABLE_SIZE, DEFAULT_LOAD_FACTOR );
    }

    /**
     * Construct the hash table.
     * @param size the approximate initial size.
     */
    public QuadraticProbingHashTableMap( int size ) {
        this( size, DEFAULT_LOAD_FACTOR );
    }

    /**
     * Construct the hash table.
     * @param size the approximate initial size.
	 * @param factor the initial load factor.
     */
    public QuadraticProbingHashTableMap( int size, double factor ) {
	}


    /**
     * Insert into the hash table. If the key of the item is
     * already in the hashtable, update its value with the new one.
	 * rehash (expand table) if it is too full (according to load factor)
	 * @param k the key of the item to insert.
	 * @param val the value to associate with the key.
     */
    public void put( Comparable k, Object val ) {
        // Insert item 
   
        // Rehash if load factor goes beyond maxloadFactor
    }

    /**
     * Expand the hash table. New size of hashtable will
	 * be about twice the size of the previous hash table. 
     */
    private void rehash( ) {
        // Create a new empty table whose size is about twice the
		// previous size, but still a prime number. 
		int newSize = nextPrime( 2 * data.length )

        // Copy items from small table into new, larger table


        return;
    }

  
    /**
     * Find a value associated with a key in the hash table.
     * @param key the key to search for.
     * @return the element associated with the matching key.
     */
    public Object get( Comparable key ) {
	   return null;
      }

    /**
     * Make the hash table logically empty.
     */
    public void makeEmpty( ) {
    }


	private static final double DEFAULT_LOAD_FACTOR = 0.5;
    private static final int DEFAULT_TABLE_SIZE = 11;

    /** instance variables here.*/
	double maxLoadFactor; // loadFactor. 
	int count;            // count of number of items in hash table
	HashEntry [] data;    // Table for storing the data
  	
    /**
     * Internal method to allocate array.
     * @param arraySize the size of the array.
     */
    private void allocateArray( int arraySize ) {
    }

    /**
     * Internal method to find a prime number at least as large as n.
     * @param n the starting number (must be positive).
     * @return a prime number larger than or equal to n.
     */
    private static int nextPrime( int n ) {
        if( n % 2 == 0 )
            n++;

        for( ; !isPrime( n ); n += 2 )
            ;

        return n;
    }

    /**
     * Internal method to test if a number is prime.
     * Not an efficient algorithm.
     * @param n the number to test.
     * @return the result of the test.
     */
    private static boolean isPrime( int n ) {
        if( n == 2 || n == 3 )
            return true;

        if( n == 1 || n % 2 == 0 )
            return false;

        for( int i = 3; i * i <= n; i += 2 )
            if( n % i == 0 )
                return false;

        return true;
    }


    // Simple main
    public static void main( String [ ] args ) {
        QuadraticProbingHashTableMap H = new QuadraticProbingHashTableMap( );

        final int NUMS = 4000;
        final int GAP  =   37;

        System.out.println( "Checking... (no more output means success)" );

        for( int i = GAP; i != 0; i = ( i + GAP ) % NUMS )
            H.put( new Integer( i ), new Integer(i) );
 
        for( int i = 2; i < NUMS; i+=2 )
            if( ((Integer)(H.get( new Integer( i ) ))).intValue( ) != i )
                System.out.println( "Find fails " + i );

    }

}